"abyss       : Black/dark blue to lightblue for bathymetry [R=-8000/0, C=RGB]",
"bathy       : Like abyss but via aquamarine at mid-depths [R=-8000/0, C=RGB]",
"categorical : Color table particularly suitable for categorical data [vRGB]",
"cool        : Linear change from blue to magenta [C=RGB]",
"copper      : Dark to light copper brown [C=RGB]",
"cubhelix    : Intensity colormap via cube helix by Dave Green [C=RGB]",
"cyclic      : Cyclic colormap, spans 360 degrees of hue [C=HSV]",
"dem1        : Digital Elevation Model (DEM) scale by Thomas Dewez [R=0/800, C=RGB]",
"dem2        : Digital Elevation Model (DEM) scale by Dewez/Wessel [R=0/4900, C=RGB]",
"dem3        : Digital Elevation Model (DEM) scale by Paul Wessel [R=0/6000, C=RGB]",
"dem4        : Digital Elevation Model (DEM) scale for Wikipedia figures [R=0/1500, C=RGB]",
"drywet      : Goes from dry to wet colors [C=RGB]",
"earth       : Colors for global bathymetry/topography relief [R=-11000/9000, H=0, C=RGB]",
"elevation   : Washed-out colors for topography [R=0/7000, C=RGB]",
"etopo1      : Colormap used in the ETOPO1 global relief map [R=-11000/8500, H=0, C=RGB]",
"gebco       : Colors for GEBCO bathymetric charts [R=-7000/0, C=RGB]",
"geo         : Colors for global bathymetry/topography relief [R=-8000/8000, H=0, C=RGB]",
"globe       : Colors for global bathymetry/topography relief [R=-10000/10000, H=0, C=RGB]",
"gray        : Gray linear ramp from black to white [C=RGB]",
"haxby       : Bill Haxby's color scheme for geoid & gravity [C=RGB]",
"hot         : Black through red and yellow to white [C=RGB]",
"ibcso       : The IBCSO bathymetry colors [R=-12000/0, C=RGB]",
"inferno     : New colormap Option B from matplotlib [C=RGB]",
"jet         : Dark to light blue, white, yellow and red [C=RGB]",
"magma       : New colormap Option A from matplotlib [C=RGB]",
"nighttime   : Colors for DMSP-OLS Nighttime Lights Time Series [C=HSV]",
"no_green    : For those who hate green [H=0, C=RGB]",
"ocean       : White-green-blue bathymetry scale [R=-8000/0, C=RGB]",
"paired      : Qualitative color map with 6 pairs of colors [C=RGB]",
"panoply     : Default colormap of Panoply [C=RGB]",
"plasma      : New colormap Option C from matplotlib [C=RGB]",
"polar       : Blue via white to red [H=0, C=RGB]",
"rainbow     : Rainbow: magenta-blue-cyan-green-yellow-red [C=HSV]",
"red2green   : Polar scale from red to green via white [H=0, C=RGB]",
"relief      : Wessel/Martinez colors for topography [R=-8000/+8000, H=0, C=RGB]",
"seafloor    : Purple-blue-white bathymetry scale [R=-6000/0, C=RGB]",
"sealand     : Smith bathymetry/topography scale [R=-6000/+3000, H=0, C=HSV]",
"seis        : R-O-Y-G-B seismic tomography colors [C=RGB]",
"split       : Like polar, but via black instead of white [H=0, C=RGB]",
"terra       : Colors for global bathymetry/topography relief [R=-7000/7000, H=0, C=RGB]",
"topo        : Sandwell/Anderson colors for topography [R=-7000/+7000, H=0, C=HSV]",
"viridis     : New colormap Option D from matplotlib [C=RGB]",
"world       : Colors for global bathymetry/topography relief [R=-7000/7000, H=0, C=RGB]",
"wysiwyg     : 20 well-separated RGB colors [C=RGB]",
