"/ISO-8859-4_Encoding [\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/space		/exclam		/quotedbl	/numbersign	/dollar		/percent	/ampersand	/quoteright\n\
/parenleft	/parenright	/asterisk	/plus		/comma		/hyphen		/period		/slash\n\
/zero		/one		/two		/three		/four		/five		/six		/seven\n\
/eight		/nine		/colon		/semicolon	/less		/equal		/greater	/question\n\
/at		/A		/B		/C		/D		/E		/F		/G\n\
/H		/I		/J		/K		/L		/M		/N		/O\n\
/P		/Q		/R		/S		/T		/U		/V		/W\n\
/X		/Y		/Z		/bracketleft	/backslash	/bracketright	/asciicircum	/underscore\n\
/quoteleft	/a		/b		/c		/d		/e		/f		/g\n\
/h		/i		/j		/k		/l		/m		/n		/o\n\
/p		/q		/r		/s		/t		/u		/v		/w\n\
/x		/y		/z		/braceleft	/bar		/braceright	/tilde		/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
/space		/Aogonek	/kra		/Rcedilla	/currency	/Itilde		/Lcedilla	/section\n\
/dieresis	/Scaron		/Emacron	/Gcedilla	/Tbar		/hyphen		/Zcaron		/macron\n\
/degree		/aogonek	/ogonek		/rcedilla	/acute		/itilde		/lcedilla	/caron\n\
/cedilla	/scaron		/emacron	/gcedilla	/tbar		/Eng		/zcaron		/eng\n\
/Amacron	/Aacute		/Acircumflex	/Atilde		/Adieresis	/Aring		/AE		/Iogonek\n\
/Ccaron		/Eacute		/Eogonek	/Edieresis	/Edotaccent	/Iacute		/Icircumflex	/Imacron\n\
/Eth		/Ncedilla	/Omacron	/Kcedilla	/Ocircumflex	/Otilde		/Odieresis	/multiply\n\
/Oslash		/Uogonek	/Uacute		/Ucircumflex	/Udieresis	/Utilde		/Umacron	/germandbls\n\
/amacron	/aacute		/acircumflex	/atilde		/adieresis	/aring		/ae		/iogonek\n\
/ccaron		/eacute		/eogonek	/edieresis	/edotaccent	/iacute		/icircumflex	/imacron\n\
/dbar		/ncedilla	/omacron	/kcedilla	/ocircumflex	/otilde		/odieresis	/divide\n\
/oslash		/uogonek	/uacute		/ucircumflex	/udieresis	/utilde		/umacron	/dotaccent\n\
] def\n",
